# Autogenerated by nbdev

d = { 'settings': { 'branch': 'main',
                'doc_baseurl': '/pote',
                'doc_host': 'https://civvic.github.io',
                'git_url': 'https://github.com/civvic/pote',
                'lib_path': 'pote'},
  'syms': { 'pote.all': {},
            'pote.basic': { 'pote.basic.AD': ('basic.html#ad', 'pote/basic.py'),
                            'pote.basic.AD.__dir__': ('basic.html#ad.__dir__', 'pote/basic.py'),
                            'pote.basic.AD.__getattr__': ('basic.html#ad.__getattr__', 'pote/basic.py'),
                            'pote.basic.AD.__setattr__': ('basic.html#ad.__setattr__', 'pote/basic.py'),
                            'pote.basic.AD._repr_markdown_': ('basic.html#ad._repr_markdown_', 'pote/basic.py'),
                            'pote.basic.AD.copy': ('basic.html#ad.copy', 'pote/basic.py'),
                            'pote.basic.Empty': ('basic.html#empty', 'pote/basic.py'),
                            'pote.basic.EmptyT': ('basic.html#emptyt', 'pote/basic.py'),
                            'pote.basic.Kounter': ('basic.html#kounter', 'pote/basic.py'),
                            'pote.basic.Kounter.__call__': ('basic.html#kounter.__call__', 'pote/basic.py'),
                            'pote.basic.Kounter.__init__': ('basic.html#kounter.__init__', 'pote/basic.py'),
                            'pote.basic.Runner': ('basic.html#runner', 'pote/basic.py'),
                            'pote.basic.WithCounterMeta': ('basic.html#withcountermeta', 'pote/basic.py'),
                            'pote.basic.WithCounterMeta.__call__': ('basic.html#withcountermeta.__call__', 'pote/basic.py'),
                            'pote.basic.WithCounterMeta.__new__': ('basic.html#withcountermeta.__new__', 'pote/basic.py'),
                            'pote.basic._get_globals': ('basic.html#_get_globals', 'pote/basic.py'),
                            'pote.basic._vals_at': ('basic.html#_vals_at', 'pote/basic.py'),
                            'pote.basic._vals_atpath': ('basic.html#_vals_atpath', 'pote/basic.py'),
                            'pote.basic.bundle_path': ('basic.html#bundle_path', 'pote/basic.py'),
                            'pote.basic.deep_in': ('basic.html#deep_in', 'pote/basic.py'),
                            'pote.basic.flatten': ('basic.html#flatten', 'pote/basic.py'),
                            'pote.basic.gets': ('basic.html#gets', 'pote/basic.py'),
                            'pote.basic.has_key': ('basic.html#has_key', 'pote/basic.py'),
                            'pote.basic.has_path': ('basic.html#has_path', 'pote/basic.py'),
                            'pote.basic.id_gen': ('basic.html#id_gen', 'pote/basic.py'),
                            'pote.basic.is_empty': ('basic.html#is_empty', 'pote/basic.py'),
                            'pote.basic.is_listy': ('basic.html#is_listy', 'pote/basic.py'),
                            'pote.basic.is_listy_type': ('basic.html#is_listy_type', 'pote/basic.py'),
                            'pote.basic.pops_': ('basic.html#pops_', 'pote/basic.py'),
                            'pote.basic.pops_values_': ('basic.html#pops_values_', 'pote/basic.py'),
                            'pote.basic.setattrs': ('basic.html#setattrs', 'pote/basic.py'),
                            'pote.basic.shorten': ('basic.html#shorten', 'pote/basic.py'),
                            'pote.basic.shortens': ('basic.html#shortens', 'pote/basic.py'),
                            'pote.basic.simple_id': ('basic.html#simple_id', 'pote/basic.py'),
                            'pote.basic.update_': ('basic.html#update_', 'pote/basic.py'),
                            'pote.basic.val_at': ('basic.html#val_at', 'pote/basic.py'),
                            'pote.basic.val_atpath': ('basic.html#val_atpath', 'pote/basic.py'),
                            'pote.basic.vals_at': ('basic.html#vals_at', 'pote/basic.py'),
                            'pote.basic.vals_atpath': ('basic.html#vals_atpath', 'pote/basic.py')},
            'pote.callback': { 'pote.callback.Callback': ('callback.html#callback', 'pote/callback.py'),
                               'pote.callback.CollBack': ('callback.html#collback', 'pote/callback.py'),
                               'pote.callback.CollBack.__init__': ('callback.html#collback.__init__', 'pote/callback.py'),
                               'pote.callback.CollBack.__iter__': ('callback.html#collback.__iter__', 'pote/callback.py'),
                               'pote.callback.CollBack.__repr__': ('callback.html#collback.__repr__', 'pote/callback.py'),
                               'pote.callback.CollBack._interrupt': ('callback.html#collback._interrupt', 'pote/callback.py'),
                               'pote.callback.CollBack._start': ('callback.html#collback._start', 'pote/callback.py'),
                               'pote.callback.CollBack._stop': ('callback.html#collback._stop', 'pote/callback.py'),
                               'pote.callback.CollBack.progress': ('callback.html#collback.progress', 'pote/callback.py'),
                               'pote.callback.CollBack.state': ('callback.html#collback.state', 'pote/callback.py'),
                               'pote.callback.CollBack.trackback': ('callback.html#collback.trackback', 'pote/callback.py'),
                               'pote.callback.EchoCB': ('callback.html#echocb', 'pote/callback.py'),
                               'pote.callback.EchoCB.__getattr__': ('callback.html#echocb.__getattr__', 'pote/callback.py'),
                               'pote.callback.EchoCB.echo': ('callback.html#echocb.echo', 'pote/callback.py'),
                               'pote.callback.FuncCB': ('callback.html#funccb', 'pote/callback.py'),
                               'pote.callback.FuncCB.__init__': ('callback.html#funccb.__init__', 'pote/callback.py'),
                               'pote.callback.HasCallbacks': ('callback.html#hascallbacks', 'pote/callback.py'),
                               'pote.callback.HasCallbacks.__getattr__': ('callback.html#hascallbacks.__getattr__', 'pote/callback.py'),
                               'pote.callback.HasCallbacks.__init__': ('callback.html#hascallbacks.__init__', 'pote/callback.py'),
                               'pote.callback.HasCallbacks.__new__': ('callback.html#hascallbacks.__new__', 'pote/callback.py'),
                               'pote.callback.HasCallbacks.callback': ('callback.html#hascallbacks.callback', 'pote/callback.py'),
                               'pote.callback.HasCallbacks.this_cbs': ('callback.html#hascallbacks.this_cbs', 'pote/callback.py'),
                               'pote.callback.HasCallbacks.with_cbs': ('callback.html#hascallbacks.with_cbs', 'pote/callback.py'),
                               'pote.callback.PassCB': ('callback.html#passcb', 'pote/callback.py'),
                               'pote.callback.PassCB.__getattr__': ('callback.html#passcb.__getattr__', 'pote/callback.py'),
                               'pote.callback.PassCB.noop': ('callback.html#passcb.noop', 'pote/callback.py'),
                               'pote.callback._get_total': ('callback.html#_get_total', 'pote/callback.py'),
                               'pote.callback.process_': ('callback.html#process_', 'pote/callback.py'),
                               'pote.callback.run_cbs': ('callback.html#run_cbs', 'pote/callback.py'),
                               'pote.callback.trackback': ('callback.html#trackback', 'pote/callback.py'),
                               'pote.callback.with_cbs': ('callback.html#with_cbs', 'pote/callback.py'),
                               'pote.callback.with_cbs.__call__': ('callback.html#with_cbs.__call__', 'pote/callback.py'),
                               'pote.callback.with_cbs.__init__': ('callback.html#with_cbs.__init__', 'pote/callback.py')},
            'pote.common': {},
            'pote.config': { 'pote.config.Config': ('config.html#config', 'pote/config.py'),
                             'pote.config.Config.__call__': ('config.html#config.__call__', 'pote/config.py'),
                             'pote.config.Config.__repr__': ('config.html#config.__repr__', 'pote/config.py'),
                             'pote.config.Config.as_dict': ('config.html#config.as_dict', 'pote/config.py'),
                             'pote.config.Config.update': ('config.html#config.update', 'pote/config.py')},
            'pote.display': { 'pote.display.GlobalCSS': ('display.html#globalcss', 'pote/display.py'),
                              'pote.display.GlobalCSS.__init__': ('display.html#globalcss.__init__', 'pote/display.py'),
                              'pote.display.GlobalCSS._html_widget': ('display.html#globalcss._html_widget', 'pote/display.py'),
                              'pote.display.GlobalCSS._ipython_display_': ('display.html#globalcss._ipython_display_', 'pote/display.py'),
                              'pote.display.GlobalCSS.add': ('display.html#globalcss.add', 'pote/display.py'),
                              'pote.display.GlobalCSS.css': ('display.html#globalcss.css', 'pote/display.py'),
                              'pote.display.GlobalCSS.display': ('display.html#globalcss.display', 'pote/display.py'),
                              'pote.display.GlobalCSS.has_style': ('display.html#globalcss.has_style', 'pote/display.py'),
                              'pote.display.GlobalCSS.update': ('display.html#globalcss.update', 'pote/display.py'),
                              'pote.display.RenderJSON': ('display.html#renderjson', 'pote/display.py'),
                              'pote.display.RenderJSON.__init__': ('display.html#renderjson.__init__', 'pote/display.py'),
                              'pote.display.RenderJSON._ipython_display_': ('display.html#renderjson._ipython_display_', 'pote/display.py'),
                              'pote.display.RenderJSON.display': ('display.html#renderjson.display', 'pote/display.py'),
                              'pote.display.cssmap': ('display.html#cssmap', 'pote/display.py'),
                              'pote.display.pretty_repr': ('display.html#pretty_repr', 'pote/display.py'),
                              'pote.display.rich_display': ('display.html#rich_display', 'pote/display.py')},
            'pote.project': { 'pote.project.get_console': ('project.html#get_console', 'pote/project.py'),
                              'pote.project.get_theme': ('project.html#get_theme', 'pote/project.py'),
                              'pote.project.setup_console': ('project.html#setup_console', 'pote/project.py')},
            'pote.test': { 'pote.test.test_afail': ('test.html#test_afail', 'pote/test.py'),
                           'pote.test.test_is_not': ('test.html#test_is_not', 'pote/test.py'),
                           'pote.test.test_raises': ('test.html#test_raises', 'pote/test.py')},
            'pote.widget': { 'pote.widget.Clickable': ('widgets.html#clickable', 'pote/widget.py'),
                             'pote.widget.Clickable.__init__': ('widgets.html#clickable.__init__', 'pote/widget.py'),
                             'pote.widget.cleanupwidgets': ('widgets.html#cleanupwidgets', 'pote/widget.py'),
                             'pote.widget.close_widget': ('widgets.html#close_widget', 'pote/widget.py'),
                             'pote.widget.get_user_input': ('widgets.html#get_user_input', 'pote/widget.py'),
                             'pote.widget.wait_for_change': ('widgets.html#wait_for_change', 'pote/widget.py'),
                             'pote.widget.yield_for_change': ('widgets.html#yield_for_change', 'pote/widget.py')}}}
